@@ -2,11 +2,13 @@
 import pytest
 
 from dragonfly.room2d import Room2D
+from dragonfly.story import Story
 from dragonfly.windowparameter import SimpleWindowRatio, SingleWindow
 from dragonfly.shadingparameter import Overhang
 
 from honeybee.boundarycondition import Outdoors, Ground, Surface
 from honeybee.boundarycondition import boundary_conditions as bcs
+from honeybee.room import Room
 
 from ladybug_geometry.geometry2d.pointvector import Point2D, Vector2D
 from ladybug_geometry.geometry2d.line import LineSegment2D
@@ -507,7 +509,7 @@ def test_to_honeybee():
     window = (ashrae_base, None, ashrae_base, None)
     shading = (overhang, None, None, None)
     room2d = Room2D('ZoneSHOE_BOX920980', Face3D(pts), 3, boundarycs, window, shading)
-    room, adj = room2d.to_honeybee(1, 0.1)
+    room, adj = room2d.to_honeybee(1, tolerance=0.1)
 
     assert room.identifier == 'ZoneSHOE_BOX920980'
     assert room.display_name == 'ZoneSHOE_BOX920980'
@@ -527,6 +529,182 @@ def test_to_honeybee():
     assert len(room[3].outdoor_shades) == 0
 
 
+def test_honeybee_ceiling_plenum():
+    """Test the add_plenum functionality in the to_honeybee method with ceiling."""
+
+    # Simple 10 x 10 room
+    tol = 0.01
+    pts1 = (Point3D(0, 0, 0), Point3D(10, 0, 0), Point3D(10, 10, 0), Point3D(0, 10, 0))
+    pts2 = (Point3D(10, 0, 0), Point3D(20, 0, 0), Point3D(20, 10, 0), Point3D(10, 10, 0))
+
+    # Two rooms with different heights
+    room2d_3m = Room2D('R1-3m', floor_geometry=Face3D(pts1), floor_to_ceiling_height=3,
+                       is_ground_contact=True, is_top_exposed=True)
+    room2d_2m = Room2D('R2-2m', floor_geometry=Face3D(pts2), floor_to_ceiling_height=2,
+                       is_ground_contact=True, is_top_exposed=True)
+
+    # Check raise exception when no Story is set
+    with pytest.raises(AttributeError):
+        _ = room2d_2m.duplicate().to_honeybee(tolerance=tol, add_plenum=True)
+
+    # Intersection at:
+    #   ((10, 0, 0), (10, 10, 0)) # room2d_3m adj idx @ 1
+    #   ((10, 10, 0), (10, 0, 0)) # room2d_2m adj idx @ 3
+
+    story = Story('S1', [room2d_3m, room2d_2m], floor_to_floor_height=3.0)
+    story.solve_room_2d_adjacency(0.01)
+
+    # Check default ceiling condition w/o plenum for 2m
+    _hb_room_2m, _ = room2d_2m.duplicate().to_honeybee(tolerance=tol, add_plenum=False)
+    assert isinstance(_hb_room_2m[-1].boundary_condition, Outdoors)
+
+    # Make HB room w/ plenum for 2m
+    hb_room_2m, _ = room2d_2m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert isinstance(hb_room_2m[0][-1].boundary_condition, Surface)
+    assert len(hb_room_2m) == 2
+
+    plenum_2m = hb_room_2m[1]
+    assert len(plenum_2m[:]) == 6
+
+    for i, face in enumerate(plenum_2m.faces):
+        if face.identifier == 'R2-2m_ceiling_plenum..Face0':
+            assert isinstance(face.boundary_condition, Surface)
+        elif face.identifier == 'R2-2m_ceiling_plenum..Face1':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-2m_ceiling_plenum..Face2':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-2m_ceiling_plenum..Face3':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-2m_ceiling_plenum..Face4':
+            assert _is_adiabatic(face.boundary_condition)
+        elif face.identifier == 'R2-2m_ceiling_plenum..Face5':
+            assert isinstance(face.boundary_condition, Outdoors) # Roof exposed outdoors
+        else:
+            assert False
+
+    # Check height of plenum
+    test_vert_face = (Point3D(10, 0, 2), Point3D(20, 0, 2),
+                      Point3D(20, 0, 3), Point3D(10, 0, 3))
+    for test_vert, vert in zip(test_vert_face, plenum_2m[1].vertices):
+        assert test_vert.is_equivalent(vert, tol)
+
+    # Make HB room w/ plenum for 3m, no plenum produced
+    hb_rooms_3m, _ = room2d_3m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert len(hb_rooms_3m) == 1
+    assert isinstance(hb_rooms_3m[0][-1].boundary_condition, Outdoors)
+
+
+def test_honeybee_floor_plenum():
+    """Test the add_plenum functionality in the to_honeybee method with floor."""
+
+    # Simple 10 x 10 room
+    tol = 0.01
+    pts1 = (Point3D(0, 0, 1), Point3D(10, 0, 1), Point3D(10, 10, 1), Point3D(0, 10, 1))
+    pts2 = (Point3D(10, 0, 1.5), Point3D(20, 0, 1.5), Point3D(20, 10, 1.5),
+            Point3D(10, 10, 1.5))
+
+    # Two rooms with different floor heights
+    room2d_1m = Room2D('R1-1m', floor_geometry=Face3D(pts1), floor_to_ceiling_height=3,
+                       is_ground_contact=True, is_top_exposed=True)
+    room2d_5m = Room2D('R2-5m', floor_geometry=Face3D(pts2), floor_to_ceiling_height=3,
+                       is_ground_contact=False, is_top_exposed=True)
+
+    story = Story('S1', [room2d_1m, room2d_5m], floor_to_floor_height=3.0)
+    story.solve_room_2d_adjacency(0.01)
+
+    # Check story floor height is minimum of room floor heights
+    assert story.floor_height == pytest.approx(1, abs=1e-10)
+
+    # Check default floor condition w/o plenum
+    _hb_room_5m, _ = room2d_5m.duplicate().to_honeybee(tolerance=tol, add_plenum=False)
+    assert isinstance(_hb_room_5m[-1].boundary_condition, Outdoors)
+    assert _is_adiabatic(_hb_room_5m[0].boundary_condition)
+
+    # Make HB room w/ plenum for 2m
+    hb_room_5m, _ = room2d_5m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert len(hb_room_5m) == 2
+
+    plenum_5m = hb_room_5m[-1]
+    assert len(plenum_5m[:]) == 6
+
+    for i, face in enumerate(plenum_5m.faces):
+        #print(face.identifier, '-', face.boundary_condition)
+        if face.identifier == 'R2-5m_floor_plenum..Face0':
+            assert _is_adiabatic(face.boundary_condition)
+        elif face.identifier == 'R2-5m_floor_plenum..Face1':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-5m_floor_plenum..Face2':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-5m_floor_plenum..Face3':
+            assert isinstance(face.boundary_condition, Outdoors)
+        elif face.identifier == 'R2-5m_floor_plenum..Face4':
+            assert _is_adiabatic(face.boundary_condition)
+        elif face.identifier == 'R2-5m_floor_plenum..Face5':
+            assert isinstance(face.boundary_condition, Surface)
+        else:
+            assert False
+
+    # Check height of plenum
+    test_vert_face = (Point3D(10, 0, 1), Point3D(20, 0, 1),
+                      Point3D(20, 0, 1.5), Point3D(10, 0, 1.5))
+    for test_vert, vert in zip(test_vert_face, plenum_5m[1].vertices):
+        assert test_vert.is_equivalent(vert, tol)
+
+    # Make HB room w/ plenum for 1m floor height, no plenum produced
+    hb_rooms_1m, _ = room2d_1m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert len(hb_rooms_1m) == 1
+    assert isinstance(hb_rooms_1m[0][0].boundary_condition, Ground)
+
+
+def test_honeybee_ceiling_and_floor_plenum():
+    """Test the add_plenum in the to_honeybee method with ceiling and floor."""
+
+    # Simple 10 x 10 room
+    tol = 0.01
+    pts1 = (Point3D(0, 0, 1), Point3D(10, 0, 1), Point3D(10, 10, 1), Point3D(0, 10, 1))
+    pts2 = (Point3D(10, 0, 1.5), Point3D(20, 0, 1.5), Point3D(20, 10, 1.5),
+            Point3D(10, 10, 1.5))
+
+    # Two rooms that require plenums
+
+    # floor_plenum: 0m, ceiling_plenum: 1m
+    room2d_1m = Room2D('R1-1m', floor_geometry=Face3D(pts1), floor_to_ceiling_height=3,
+                       is_ground_contact=True, is_top_exposed=False)
+    # floor_plenum: 0.5m, ceiling_plenum: 1m
+    room2d_5m = Room2D('R2-5m', floor_geometry=Face3D(pts2), floor_to_ceiling_height=3,
+                       is_ground_contact=False, is_top_exposed=False)
+
+    story = Story('S1', [room2d_1m, room2d_5m], floor_to_floor_height=4.0)
+    story.solve_room_2d_adjacency(tol)
+
+    # Make HB room w/ just ceiling plenum
+    hb_rooms_1m, _ = room2d_1m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert len(hb_rooms_1m) == 2
+
+    # Make HB room w/ both
+    hb_rooms_5m, _ = room2d_5m.to_honeybee(tolerance=tol, add_plenum=True)
+    assert len(hb_rooms_5m) == 3
+
+    hb_room_5m, ceil_plenum_5m, floor_plenum_5m = hb_rooms_5m
+
+    # Check names
+    assert hb_room_5m.identifier == 'R2-5m'
+    assert ceil_plenum_5m.identifier == 'R2-5m_ceiling_plenum'
+    assert floor_plenum_5m.identifier == 'R2-5m_floor_plenum'
+
+    # Check height of floor_plenum
+    test_vert_face = (Point3D(10, 0, 1), Point3D(20, 0, 1),
+                      Point3D(20, 0, 1.5), Point3D(10, 0, 1.5))
+    for test_vert, vert in zip(test_vert_face, floor_plenum_5m[1].vertices):
+        assert test_vert.is_equivalent(vert, tol)
+
+    # Check height of ceil_plenum
+    test_vert_face = (Point3D(10, 0, 4.5), Point3D(20, 0, 4.5),
+                      Point3D(20, 0, 5.0), Point3D(10, 0, 5.0))
+    for test_vert, vert in zip(test_vert_face, ceil_plenum_5m[1].vertices):
+        assert test_vert.is_equivalent(vert, tol)
+
+
 def test_to_dict():
     """Test the Room2D to_dict method."""
     pts = (Point3D(0, 0, 3), Point3D(5, 0, 3), Point3D(5, 10, 3), Point3D(0, 10, 3))
@@ -589,3 +767,13 @@ def test_writer():
     writers = [mod for mod in dir(room2d.to) if not mod.startswith('_')]
     for writer in writers:
         assert callable(getattr(room2d.to, writer))
+
+
+def _is_adiabatic(bc):
+    """Test if adiabatic instance, or if honeybee-energy not installed,
+    if using default Outdoors.
+    """
+    try:
+        return isinstance(bc, type(bcs.adiabatic))
+    except AttributeError:
+        return isinstance(bc, Outdoors)
